@@ -4,6 +4,23 @@ void get_system_name()
 {
 	FILE *f;
 	unsigned char *p;
+#if defined(HAVE_SYS_UTSNAME_H) && defined(HAVE_UNAME)
+	struct utsname name;
+	memset(&name, 0, sizeof name);
+	if (!uname(&name)) {
+		unsigned char *str = init_str();
+		int l = 0;
+		add_to_str(&str, &l, name.sysname);
+		add_to_str(&str, &l, " ");
+		add_to_str(&str, &l, name.release);
+		add_to_str(&str, &l, " ");
+		add_to_str(&str, &l, name.machine);
+		if (l >= MAX_STR_LEN) str[MAX_STR_LEN - 1] = 0;
+		strcpy(system_name, str);
+		mem_free(str);
+		return;
+	}
+#endif
 #ifdef HAVE_POPEN
 	memset(system_name, 0, MAX_STR_LEN);
 	if (!(f = popen("uname -srm", "r"))) goto fail;
@@ -570,7 +587,7 @@ unsigned char *term_rd(struct option *o, unsigned char *c)
 	}
 	mem_free(w);
 	if (!(w = get_token(&c))) goto err;
-	if (strlen(w) != 1 || w[0] < '0' || w[0] > '3') goto err_f;
+	if (strlen(w) != 1 || w[0] < '0' || w[0] > '4') goto err_f;
 	ts->mode = w[0] - '0';
 	mem_free(w);
 	if (!(w = get_token(&c))) goto err;
@@ -607,7 +624,7 @@ unsigned char *term2_rd(struct option *o, unsigned char *c)
 	}
 	mem_free(w);
 	if (!(w = get_token(&c))) goto err;
-	if (strlen(w) != 1 || w[0] < '0' || w[0] > '3') goto err_f;
+	if (strlen(w) != 1 || w[0] < '0' || w[0] > '4') goto err_f;
 	ts->mode = w[0] - '0';
 	mem_free(w);
 	if (!(w = get_token(&c))) goto err;
@@ -875,7 +892,7 @@ unsigned char download_dir[MAX_STR_LEN] = "";
 unsigned char default_anon_pass[MAX_STR_LEN] = "somebody@host.domain";
 
 /* These are workarounds for some CGI script bugs */
-struct http_bugs http_bugs = { 0, 1, 1, 0 };
+struct http_bugs http_bugs = { 0, 1, 1, 0, 0 };
 /*int bug_302_redirect = 0;*/
 	/* When got 301 or 302 from POST request, change it to GET
 	   - this violates RFC2068, but some buggy message board scripts rely on it */
@@ -898,9 +915,9 @@ struct option links_options[] = {
 	1, dump_cmd, NULL, NULL, D_SOURCE, 0, NULL, NULL, "source",
 	1, gen_cmd, num_rd, num_wr, 0, 1, &async_lookup, "async_dns", "async-dns",
 	1, gen_cmd, num_rd, num_wr, 0, 1, &download_utime, "download_utime", "download-utime",
-	1, gen_cmd, num_rd, num_wr, 1, 16, &max_connections, "max_connections", "max-connections",
-	1, gen_cmd, num_rd, num_wr, 1, 8, &max_connections_to_host, "max_connections_to_host", "max-connections-to-host",
-	1, gen_cmd, num_rd, num_wr, 1, 16, &max_tries, "retries", "retries",
+	1, gen_cmd, num_rd, num_wr, 1, 99, &max_connections, "max_connections", "max-connections",
+	1, gen_cmd, num_rd, num_wr, 1, 99, &max_connections_to_host, "max_connections_to_host", "max-connections-to-host",
+	1, gen_cmd, num_rd, num_wr, 0, 16, &max_tries, "retries", "retries",
 	1, gen_cmd, num_rd, num_wr, 1, 1800, &receive_timeout, "receive_timeout", "receive-timeout",
 	1, gen_cmd, num_rd, num_wr, 1, 1800, &unrestartable_receive_timeout, "unrestartable_receive_timeout", "unrestartable-receive-timeout",
 	1, gen_cmd, num_rd, NULL, 10, 512, &screen_width, NULL, "width",
@@ -913,8 +930,11 @@ struct option links_options[] = {
 	1, gen_cmd, num_rd, num_wr, 0, 1, &http_bugs.http10, "http_bugs.http10", "http-bugs.http10",
 	1, gen_cmd, num_rd, num_wr, 0, 1, &http_bugs.allow_blacklist, "http_bugs.allow_blacklist", "http-bugs.allow-blacklist",
 	1, gen_cmd, num_rd, num_wr, 0, 1, &http_bugs.bug_302_redirect, "http_bugs.bug_302_redirect", "http-bugs.bug-302-redirect",
-	1, gen_cmd, num_rd, num_wr, 0, 1, &http_bugs.bug_post_no_keepalive, "http_bugs.bug_post_no_keepalive", "http-bugs.bug_post-no-keepalive",
+	1, gen_cmd, num_rd, num_wr, 0, 1, &http_bugs.bug_post_no_keepalive, "http_bugs.bug_post_no_keepalive", "http-bugs.bug-post-no-keepalive",
+	1, gen_cmd, num_rd, num_wr, 0, 1, &http_bugs.no_accept_charset, "http_bugs.no_accept_charset", "http-bugs.bug-no-accept-charset",
 	1, gen_cmd, str_rd, str_wr, 0, MAX_STR_LEN, default_anon_pass, "ftp.anonymous_password", "ftp.anonymous-password",
+	{1, gen_cmd, num_rd, num_wr, 0, 1, &passive_ftp, "ftp.use_passive", "ftp.use-passive"},
+	{1, gen_cmd, num_rd, num_wr, 0, 1, &fast_ftp, "ftp.fast", "ftp.fast"},
 	1, gen_cmd, cp_rd, NULL, 0, 0, &dds.assume_cp, "assume_codepage", "assume-codepage",
 	1, NULL, term_rd, term_wr, 0, 0, NULL, "terminal", NULL,
 	1, NULL, term2_rd, NULL, 0, 0, NULL, "terminal2", NULL,
@@ -963,7 +983,8 @@ int load_url_history()
 	}
 	while (fgets (url, MAX_INPUT_URL_LEN, fp))
 	{
-		url[strlen(url)-1] = 0 ;
+		url[MAX_INPUT_URL_LEN - 1] = 0;
+		if (*url) url[strlen(url)-1] = 0 ;
 		add_to_history(&goto_url_history, url) ;
 	}
 	fclose (fp) ;
