@@ -809,9 +809,11 @@ void check_table_widths(struct table *t)
 		for (k = 0; k < c->colspan; k++) p += t->w_c[i + k];
 		get_cell_width(c->start, c->end, t->cellpd, p, 1, &c->x_width, NULL, c->link_num, NULL);
 		if (c->x_width > p) {
-			int min, max;
+			/*int min, max;
 			get_cell_width(c->start, c->end, t->cellpd, 0, 0, &min, &max, c->link_num, NULL);
-			internal("cell is now wider (%d > %d) min = %d, max = %d, now_min = %d, now_max = %d", c->x_width, p, t->min_c[i], t->max_c[i], min, max);
+			internal("cell is now wider (%d > %d) min = %d, max = %d, now_min = %d, now_max = %d", c->x_width, p, t->min_c[i], t->max_c[i], min, max);*/
+			/* sbohem, internale. chytl jsi mi spoustu chyb v tabulkovaci, ale ted je proste cas jit ... ;-( */
+			c->x_width = p;
 		}
 	}
 	s = 1;
@@ -1089,6 +1091,7 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	struct s_e *bad_html;
 	int bad_html_n;
 	struct node *n, *nn;
+	int cpd_pass, cpd_width, cpd_last;
 	/*if (!p->data) {
 		debug("nested tables not supported");
 		return;
@@ -1096,9 +1099,9 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	table_level++;
 	memcpy(&bgcolor, &par_format.bgcolor, sizeof(struct rgb));
 	get_bgcolor(attr, &bgcolor);
-	if ((border = get_num(attr, "border")) == -1) border = has_attr(attr, "border");
+	if ((border = get_num(attr, "border")) == -1) border = has_attr(attr, "border") || has_attr(attr, "rules") || has_attr(attr, "frame");
 	/*if (!border) border = 1;*/
-								/* !!! FIXME: remove */
+
 	if ((cellsp = get_num(attr, "cellspacing")) == -1) cellsp = 1;
 	if ((cellpd = get_num(attr, "cellpadding")) == -1) {
 		vcellpd = 0;
@@ -1132,7 +1135,7 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 		if (!strcasecmp(al, "border")) frame = F_BOX;
 		mem_free(al);
 	}
-	rules = R_ALL;
+	rules = border ? R_ALL : R_NONE;
 	if ((al = get_attr_val(attr, "rules"))) {
 		if (!strcasecmp(al, "none")) rules = R_NONE;
 		if (!strcasecmp(al, "groups")) rules = R_GROUPS;
@@ -1145,6 +1148,7 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	wf = 0;
 	if ((width = get_width(attr, "width", p->data || p->xp)) == -1) {
 		width = par_format.width - par_format.leftmargin - par_format.rightmargin;
+		if (width < 0) width = 0;
 		wf = 1;
 	}
 	if (!(t = parse_table(html, eof, end, &bgcolor, p->data || p->xp, &bad_html, &bad_html_n))) {
@@ -1169,6 +1173,9 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	t->rules = rules;
 	t->width = width;
 	t->wf = wf;
+	cpd_pass = 0;
+	cpd_last = t->cellpd;
+	cpd_width = 0;	/* not needed, but let the warning go away */
 	again:
 	get_cell_widths(t);
 	if (get_column_widths(t)) goto ret2;
@@ -1180,8 +1187,15 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 		if (t->min_t + par_format.leftmargin + par_format.rightmargin > p->x) p->x = t->min_t + par_format.leftmargin + par_format.rightmargin;
 		goto ret2;
 	}
-	if (t->min_t > width && t->cellpd) {
+	if (!cpd_pass && t->min_t > width && t->cellpd) {
 		t->cellpd = 0;
+		cpd_pass = 1;
+		cpd_width = t->min_t;
+		goto again;
+	}
+	if (cpd_pass == 1 && t->min_t > cpd_width) {
+		t->cellpd = cpd_last;
+		cpd_pass = 2;
 		goto again;
 	}
 	/*debug("%d %d %d", t->min_t, t->max_t, width);*/
@@ -1235,8 +1249,9 @@ void format_table(unsigned char *attr, unsigned char *html, unsigned char *eof,
 	if (p->cy > p->y) p->y = p->cy;
 	/*ret1:*/
 	free_table(t);
-	ret0:
 	kill_html_stack_item(&html_top);
+	ret0:
 	/*ret:*/
 	table_level--;
+	if (!table_level) free_table_cache();
 }
