@@ -139,11 +139,11 @@ int check_http_server_bugs(unsigned char *url, struct http_connection_info *info
 	return 0;	
 }
 
-void http_end_request(struct connection *c)
+void http_end_request(struct connection *c, int notrunc)
 {
 	if (c->state == S_OK) {
 		if (c->cache) {
-			truncate_entry(c->cache, c->from, 1);
+			if (!notrunc) truncate_entry(c->cache, c->from, 1);
 			c->cache->incomplete = 0;
 		}
 	}
@@ -161,7 +161,7 @@ void http_send_header(struct connection *);
 void http_func(struct connection *c)
 {
 	/*setcstate(c, S_CONN);*/
-	set_timeout(c);
+	/*set_timeout(c);*/
 	if (get_keepalive_socket(c)) {
 		int p;
 		if ((p = get_port(c->url)) == -1) {
@@ -187,7 +187,7 @@ void http_send_header(struct connection *c)
 	int http10 = http_bugs.http10;
 	struct cache_entry *e = NULL;
 	unsigned char *hdr;
-	unsigned char *h, *u, *uu;
+	unsigned char *h, *u, *uu, *sp;
 	int l = 0;
 	unsigned char *post;
 	unsigned char *host = upcase(c->url[0]) != 'P' ? c->url : get_url_data(c->url);
@@ -209,7 +209,7 @@ void http_send_header(struct connection *c)
 	if (post) post++;
 	if (!(hdr = init_str())) {
 		setcstate(c, S_OUT_OF_MEM);
-		http_end_request(c);
+		http_end_request(c, 0);
 		return;
 	}
 	if (!post) add_to_str(&hdr, &l, "GET ");
@@ -219,23 +219,30 @@ void http_send_header(struct connection *c)
 	}
 	if (upcase(c->url[0]) != 'P') add_to_str(&hdr, &l, "/");
 	if (!(u = get_url_data(c->url))) {
+		mem_free(hdr);
 		setcstate(c, S_BAD_URL);
-		http_end_request(c);
+		http_end_request(c, 0);
+		return;
+	}
+	if (post && post < u) {
+		mem_free(hdr);
+		setcstate(c, S_BAD_URL);
+		http_end_request(c, 0);
 		return;
 	}
 	if (!post) uu = stracpy(u);
 	else uu = memacpy(u, post - u - 1);
-	while (strchr(uu, ' ')) {
-		unsigned char *sp = strchr(uu, ' ');
+	a:
+	for (sp = uu; *sp; sp++) if (*sp <= ' ') {
 		unsigned char *nu = mem_alloc(strlen(uu) + 3);
 		if (!nu) break;
 		memcpy(nu, uu, sp - uu);
-		nu[sp - uu] = 0;
-		strcat(nu, "%20");
+		sprintf(nu + (sp - uu), "%%%02X", (int)*sp);
 		strcat(nu, sp + 1);
 		mem_free(uu);
 		uu = nu;
-	}
+		goto a;
+	} else if (*sp == '\\') *sp = '/';
 	add_to_str(&hdr, &l, uu);
 	mem_free(uu);
 	if (!http10) add_to_str(&hdr, &l, " HTTP/1.1\r\n");
@@ -277,7 +284,7 @@ void http_send_header(struct connection *c)
 		else accept_charset = "";
 		mem_free(ac);
 	}
-	if (!(info->bl_flags & BL_NO_CHARSET)) add_to_str(&hdr, &l, accept_charset);
+	if (!(info->bl_flags & BL_NO_CHARSET) && !http_bugs.no_accept_charset) add_to_str(&hdr, &l, accept_charset);
 	if (!http10) {
 		if (upcase(c->url[0]) != 'P') add_to_str(&hdr, &l, "Connection: ");
 		else add_to_str(&hdr, &l, "Proxy-Connection: ");
@@ -329,7 +336,7 @@ void http_send_header(struct connection *c)
 			post += 2;
 		}
 	}
-	write_to_socket(c, c->sock1, hdr, strlen(hdr), http_get_header);
+	write_to_socket(c, c->sock1, hdr, l, http_get_header);
 	mem_free(hdr);
 	setcstate(c, S_SENT);
 }
@@ -352,7 +359,7 @@ void read_http_data(struct connection *c, struct read_buffer *rb)
 	set_timeout(c);
 	if (rb->close == 2) {
 		setcstate(c, S_OK);
-		http_end_request(c);
+		http_end_request(c, 0);
 		return;
 	}
 	if (info->length != -2) {
@@ -365,7 +372,7 @@ void read_http_data(struct connection *c, struct read_buffer *rb)
 		kill_buffer_data(rb, l);
 		if (!info->length && !rb->close) {
 			setcstate(c, S_OK);
-			http_end_request(c);
+			http_end_request(c, 0);
 			return;
 		}
 	} else {
@@ -381,7 +388,7 @@ void read_http_data(struct connection *c, struct read_buffer *rb)
 				kill_buffer_data(rb, l);
 				if (l <= 2) {
 					setcstate(c, S_OK);
-					http_end_request(c);
+					http_end_request(c, 0);
 					return;
 				}
 				goto next_chunk;
@@ -527,15 +534,22 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 	if (h == 304) {
 		mem_free(head);
 		setcstate(c, S_OK);
-		http_end_request(c);
+		http_end_request(c, 1);
 		return;
 	}
-	if (get_cache_entry(c->url, &e)) {
+	if (h == 204) {
+		mem_free(head);
+		setcstate(c, S_OK);
+		http_end_request(c, 0);
+		return;
+	}
+	if (!c->cache && get_cache_entry(c->url, &c->cache)) {
 		mem_free(head);
 		setcstate(c, S_OUT_OF_MEM);
 		abort_connection(c);
 		return;
 	}
+	e = c->cache;
 	if (e->head) mem_free(e->head);
 	e->head = head;
 #ifdef HAVE_SSL
@@ -550,11 +564,7 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
 		add_to_str(&e->ssl_info, &l, (unsigned  char *)SSL_get_cipher_name(c->ssl));
 	}
 #endif
-	if (h == 204) {
-		setcstate(c, S_OK);
-		http_end_request(c);
-		return;
-	}
+	if (e->redirect) mem_free(e->redirect), e->redirect = NULL;
 	if (h == 301 || h == 302 || h == 303) {
 		if ((d = parse_http_header(e->head, "Location", NULL))) {
 			if (e->redirect) mem_free(e->redirect);
@@ -571,7 +581,6 @@ void http_got_header(struct connection *c, struct read_buffer *rb)
  		mem_free(d);
   	}*/
 	kill_buffer_data(rb, a);
-	c->cache = e;
 	info->close = 0;
 	info->length = -1;
 	info->version = version;
