@@ -4,6 +4,7 @@
 #define IN_BUF_SIZE	16
 
 #define USE_TWIN_MOUSE	1
+#define USE_X_MOUSE	2
 #define TW_BUTT_LEFT	1
 #define TW_BUTT_MIDDLE	2
 #define TW_BUTT_RIGHT	4
@@ -101,7 +102,7 @@ void send_init_sequence(int h,int flags)
 	hard_write(h, init_seq, strlen(init_seq));
 	if (flags & USE_TWIN_MOUSE) {
 		hard_write(h, init_seq_tw_mouse, strlen(init_seq_tw_mouse));
-	} else {
+	} else if (flags & USE_X_MOUSE) {
 		hard_write(h, init_seq_x_mouse, strlen(init_seq_x_mouse));
 	}
 }
@@ -111,7 +112,7 @@ void send_term_sequence(int h,int flags)
 	hard_write(h, term_seq, strlen(term_seq));
 	if (flags & USE_TWIN_MOUSE) {
 		hard_write(h, term_seq_tw_mouse, strlen(term_seq_tw_mouse));
-	} else {
+	} else if (flags & USE_X_MOUSE) {
 		hard_write(h, term_seq_x_mouse, strlen(term_seq_x_mouse));
 	}
 }
@@ -132,7 +133,7 @@ int setraw(int fd, struct termios *p)
 	memset(&t, 0, sizeof(struct termios));
 	if (tcgetattr(fd, &t)) return -1;
 	if (p) memcpy(p, &t, sizeof(struct termios));
-	cfmakeraw(&t);
+	os_cfmakeraw(&t);
 	t.c_lflag |= ISIG;
 #ifdef TOSTOP
 	t.c_lflag |= TOSTOP;
@@ -176,6 +177,7 @@ void handle_trm(int std_in, int std_out, int sock_in, int sock_out, int ctl_in,
 	itrm->flags = 0;
 	if (!(ts = getenv("TERM"))) ts = "";
 	if ((xwin & ENV_TWIN) && !strcmp(ts,"linux")) itrm->flags |= USE_TWIN_MOUSE;
+	if ((xwin & ENV_XWIN)) itrm->flags |= USE_X_MOUSE;
 	if (strlen(ts) >= MAX_TERM_LEN) queue_event(itrm, ts, MAX_TERM_LEN);
 	else {
 		unsigned char *mm;
@@ -490,13 +492,11 @@ int process_queue(struct itrm *itrm)
 		if (itrm->kqueue[1] == '[' || itrm->kqueue[1] == 'O') {
 			char c;
 			int v;
-			if (get_esc_code(itrm->kqueue, itrm->qlen, &c, &v, &el)) {
-				if (itrm->qlen < 4 || itrm->kqueue[2] != '[') goto ret;
-				if (itrm->kqueue[2] != '[') goto ret;
+			if (itrm->qlen >= 4 && itrm->kqueue[2] == '[') {
 				if (itrm->kqueue[3] < 'A' || itrm->kqueue[3] > 'L') goto ret;
-				ev.x = KBD_F1 + itrm->kqueue[3] - 'A';
+				ev.x = KBD_F1 - (itrm->kqueue[3] - 'A');
 				el = 4;
-			}
+			} else if (get_esc_code(itrm->kqueue, itrm->qlen, &c, &v, &el)) goto ret;
 			else switch (c) {
 				case 'A': ev.x = KBD_UP; break;
 				case 'B': ev.x = KBD_DOWN; break;
