@@ -64,6 +64,9 @@
 #ifdef HAVE_IO_H
 #include <io.h>
 #endif
+#ifdef HAVE_SYS_UTSNAME_H
+#include <sys/utsname.h>
+#endif
 /*
 #ifdef HAVE_WAIT_H
 #include <wait.h>
@@ -101,6 +104,10 @@
 
 #include "setup.h"
 
+#define LINKS_COPYRIGHT "(C) 1999 - 2003 Mikulas Patocka"
+#define LINKS_COPYRIGHT_8859_1 "(C) 1999 - 2003 Mikuls Patocka"
+#define LINKS_COPYRIGHT_8859_2 "(C) 1999 - 2003 Mikul Patoka"
+
 #define DUMMY ((void *)-1L)
 
 /*
@@ -433,16 +440,12 @@ struct list_head {
 	void *prev;
 };
 
-#ifndef HAVE_TYPEOF
-
 struct xlist_head {
 	struct xlist_head *next;
 	struct xlist_head *prev;
 };
 
-#endif
-
-#define init_list(x) {(x).next=&(x); (x).prev=&(x);}
+#define init_list(x) {do_not_optimize_here(&x); (x).next=&(x); (x).prev=&(x); do_not_optimize_here(&x);}
 #define list_empty(x) ((x).next == &(x))
 #define del_from_list(x) {do_not_optimize_here(x); ((struct list_head *)(x)->next)->prev=(x)->prev; ((struct list_head *)(x)->prev)->next=(x)->next; do_not_optimize_here(x);}
 /*#define add_to_list(l,x) {(x)->next=(l).next; (x)->prev=(typeof(x))&(l); (l).next=(x); if ((l).prev==&(l)) (l).prev=(x);}*/
@@ -458,7 +461,7 @@ struct xlist_head {
 #define foreach(e,l) for ((e)=(l).next; (e)!=(void *)&(l); (e)=(e)->next)
 #define foreachback(e,l) for ((e)=(l).prev; (e)!=(void *)&(l); (e)=(e)->prev)
 #endif
-#define free_list(l) {do_not_optimize_here(&l); while ((l).next != &(l)) {struct list_head *a__=(l).next; del_from_list(a__); mem_free(a__); } do_not_optimize_here(&l);}
+#define free_list(l) {struct xlist_head *a__;do_not_optimize_here(&l);foreach(a__,l) do_not_optimize_here(a__);foreachback(a__,l) do_not_optimize_here(a__); while ((l).next != &(l)) {a__=(l).next; del_from_list(a__); mem_free(a__); } do_not_optimize_here(&l);}
 
 #define WHITECHAR(x) ((x) == 9 || (x) == 10 || (x) == 12 || (x) == 13 || (x) == ' ')
 #define U(x) ((x) == '"' || (x) == '\'')
@@ -514,6 +517,7 @@ struct open_in_new {
 	void (*fn)(struct terminal *term, unsigned char *, unsigned char *);
 };
 
+void close_fork_tty();
 int get_system_env();
 int is_xterm();
 int can_twterm();
@@ -552,6 +556,7 @@ void prealloc_truncate(int, int);
 #else
 static inline void prealloc_truncate(int x, int y) {}
 #endif
+void os_cfmakeraw(struct termios *t);
 
 /* select.c */
 
@@ -671,6 +676,7 @@ struct connection {
 	int sock1;
 	int sock2;
 	void *dnsquery;
+	pid_t pid;
 	int tries;
 	struct list_head statuss;
 	void *info;
@@ -804,8 +810,8 @@ void add_auth_entry(unsigned char*, unsigned char *);
 void del_auth_entry(struct http_auth_basic *);
 void free_auth();
 
-#define BL_HTTP10	1
-#define BL_NO_CHARSET	2
+#define BL_HTTP10		1
+#define BL_NO_CHARSET		2
 
 /* url.c */
 
@@ -885,8 +891,14 @@ void finger_func(struct connection *);
 
 /* ftp.c */
 
+extern int fast_ftp;
+extern int passive_ftp;
 void ftp_func(struct connection *);
 
+/* smb.c */
+
+void smb_func(struct connection *);
+
 /* mailto.c */
 
 void mailto_func(struct session *, unsigned char *);
@@ -1026,6 +1038,7 @@ struct term_spec {
 #define TERM_VT100	1
 #define TERM_LINUX	2
 #define TERM_KOI8	3
+#define TERM_FREEBSD	4
 
 #define ATTR_FRAME	0x8000
 
@@ -1406,7 +1419,6 @@ struct view_state {
 	unsigned char *goto_position;
 	struct form_state *form_info;
 	int form_info_len;
-	struct f_data_c *f;
 	unsigned char url[1];
 };
 
@@ -1767,6 +1779,7 @@ unsigned char *print_current_link(struct session *);
 unsigned char *print_current_title(struct session *);
 void loc_msg(struct terminal *, struct location *, struct f_data_c *);
 void state_msg(struct session *);
+void head_msg(struct session *);
 void search_for(struct session *, unsigned char *);
 void search_for_back(struct session *, unsigned char *);
 void find_next(struct session *, struct f_data_c *, int);
@@ -2037,6 +2050,7 @@ struct http_bugs {
 	int allow_blacklist;
 	int bug_302_redirect;
 	int bug_post_no_keepalive;
+	int no_accept_charset;
 };
 
 extern struct http_bugs http_bugs;
