@@ -293,6 +293,12 @@ int is_xterm()
 
 tcount resize_count = 0;
 
+void close_fork_tty()
+{
+	struct terminal *t;
+	foreach (t, terminals) if (t->fdin > 0) close(t->fdin);
+}
+
 #if defined(UNIX) || defined(WIN32) || defined(BEOS) || defined(RISCOS)
 
 #if defined(BEOS) && defined(HAVE_SETPGID)
@@ -392,7 +398,7 @@ char *get_clipboard_text()
 	HAB hab;
 	HMQ hmq;
 	ULONG oldType;
-	char *ret = 0;
+	char *ret = NULL;
 
 	DosGetInfoBlocks(&tib, &pib);
 
@@ -413,8 +419,11 @@ char *get_clipboard_text()
 					if (selClipText)
 					{
 						PCHAR pchClipText = (PCHAR)selClipText;
-						if ((ret = mem_alloc(strlen(pchClipText)+1)))
+						if ((ret = mem_alloc(strlen(pchClipText)+1))) {
+							char *u;
 							strcpy(ret, pchClipText);
+							while ((u = strchr(ret, 13))) memmove(u, u + 1, strlen(u + 1) + 1);
+						}
 					}
 				}
 
@@ -1093,8 +1102,7 @@ int start_thread(void (*fn)(void *, int), void *ptr, int l)
 	fcntl(p[0], F_SETFL, O_NONBLOCK);
 	fcntl(p[1], F_SETFL, O_NONBLOCK);
 	if (!(f = fork())) {
-		struct terminal *t;
-		foreach (t, terminals) if (t->fdin > 0) close(t->fdin);
+		close_fork_tty();
 		close(p[0]);
 		fn(ptr, p[1]);
 		write(p[1], "x", 1);
@@ -1183,9 +1191,14 @@ int get_input_handle()
 #endif /* defined(HAVE_BEGINTHREAD) && defined(HAVE_READ_KBD) */
 
 
-#ifndef HAVE_CFMAKERAW
-void cfmakeraw(struct termios *t)
+void os_cfmakeraw(struct termios *t)
 {
+#ifdef HAVE_CFMAKERAW
+	cfmakeraw(t);
+#ifdef VMIN
+	t->c_cc[VMIN] = 1;	/* cfmakeraw is broken on AIX */
+#endif
+#else
 	t->c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
 	t->c_oflag &= ~OPOST;
 	t->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
@@ -1193,8 +1206,8 @@ void cfmakeraw(struct termios *t)
 	t->c_cflag |= CS8;
 	t->c_cc[VMIN] = 1;
 	t->c_cc[VTIME] = 0;
-}
 #endif
+}
 
 #ifdef USE_GPM
 
@@ -1311,7 +1324,7 @@ void open_in_new_twterm(struct terminal *term, unsigned char *exe, unsigned char
 void open_in_new_xterm(struct terminal *term, unsigned char *exe, unsigned char *param)
 {
 	unsigned char *xterm;
-	if (!(xterm = getenv("LINKS_XTERM"))) xterm = "xterm -e";
+	if (!(xterm = getenv("LINKS_XTERM"))) xterm = "x-terminal-emulator -e";
 	exec_new_links(term, xterm, exe, param);
 }
 
