@@ -176,6 +176,12 @@ void free_connection_data(struct connection *c)
 	if (c->sock1 != -1) set_handlers(c->sock1, NULL, NULL, NULL, NULL);
 	if (c->sock2 != -1) set_handlers(c->sock2, NULL, NULL, NULL, NULL);
 	close_socket(&c->sock2);
+	if (c->pid) {
+		kill(c->pid, SIGINT);
+		kill(c->pid, SIGTERM);
+		kill(c->pid, SIGKILL);
+		c->pid = 0;
+	}
 	if (!c->running) {
 		internal("connection already suspended");
 	}
@@ -400,7 +406,7 @@ void run_connection(struct connection *c)
 void retry_connection(struct connection *c)
 {
 	interrupt_connection(c);
-	if (c->unrestartable >= 2 || ++c->tries >= max_tries) {
+	if (c->unrestartable >= 2 || (++c->tries >= (max_tries ? max_tries : 1000))) {
 		/*send_connection_info(c);*/
 		del_connection(c);
 #ifdef DEBUG
@@ -589,7 +595,7 @@ int load_url(unsigned char *url, struct status *stat, int pri, int no_cache)
 	c->url = u;
 	c->running = 0;
 	c->prev_error = 0;
-	c->from = no_cache >= NC_RELOAD || !e || e->frag.next == &e->frag || ((struct fragment *)e->frag.next)->offset ? 0 : ((struct fragment *)e->frag.next)->length;
+	c->from = no_cache >= NC_IF_MOD || !e || e->frag.next == &e->frag || ((struct fragment *)e->frag.next)->offset ? 0 : ((struct fragment *)e->frag.next)->length;
 	memset(c->pri, 0, sizeof c->pri);
 	c->pri[pri] = 1;
 	c->no_cache = no_cache;
