@@ -1268,6 +1268,7 @@ unsigned char *encode_url(unsigned char *url)
 {
 	unsigned char *u = init_str();
 	int l = 0;
+	add_to_str(&u, &l, "links+");				/* add a unique prefix to show it's encoded */
 	for (; *url; url++) {
 		if (is_safe_in_shell(*url) && *url != '+') add_chr_to_str(&u, &l, *url);
 		else add_chr_to_str(&u, &l, '+'), add_chr_to_str(&u, &l, hx(*url >> 4)), add_chr_to_str(&u, &l, hx(*url & 0xf));
@@ -1279,6 +1280,11 @@ unsigned char *decode_url(unsigned char *url)
 {
 	unsigned char *u = init_str();
 	int l = 0;
+        if (strncmp(url, "links+", strlen("links+"))) {		/* check unique prefix [Ben Pfaff] */
+                add_to_str(&u, &l, url);
+                return u;
+        }
+        else url += strlen("links+");
 	for (; *url; url++) {
 		if (*url != '+' || unhx(url[1]) == -1 || unhx(url[2]) == -1) add_chr_to_str(&u, &l, *url);
 		else add_chr_to_str(&u, &l, (unhx(url[1]) << 4) + unhx(url[2])), url += 2;
