@@ -134,7 +134,7 @@ void redraw_from_window(struct window *win)
 	ev.x = term->x;
 	ev.y = term->y;
 	if (term->redrawing) return;
-	win->term->redrawing = 1;
+	term->redrawing = 1;
 	for (win = win->prev; win != end; win = win->prev) {
 		win->handler(win, &ev, 0);
 	}
@@ -273,7 +273,7 @@ void free_term_specs()
 
 struct list_head term_specs = {&term_specs, &term_specs};
 
-struct term_spec dumb_term = { NULL, NULL, "", 0, 0, 0, 0, 0 };
+struct term_spec dumb_term = { NULL, NULL, "", 0, 1, 0, 0, 0 };
 
 struct term_spec *get_term_spec(unsigned char *term)
 {
@@ -407,7 +407,10 @@ void in_term(struct terminal *term)
 		else if (ev->ev == EV_KBD && ev->x == KBD_CTRL_C) ((struct window *)&term->windows)->prev->handler(term->windows.prev, ev, 0);
 		else ((struct window *)&term->windows)->next->handler(term->windows.next, ev, 0);
 	}
-	if (ev->ev == EV_ABORT) destroy_terminal(term);
+	if (ev->ev == EV_ABORT) {
+		destroy_terminal(term);
+		return;
+	}
 	/*redraw_screen(term);*/
 	mm:
 	if (term->qlen == r) term->qlen = 0;
@@ -430,6 +433,14 @@ unsigned char frame_koi[48] = {
 	186,182,183,170,169,162,164,189,
 	188,133,130,141,140,142,143,139,
 };
+unsigned char frame_freebsd[48] = {
+	130,138,128,153,150,150,150,140,
+	140,150,153,140,139,139,139,140,
+	142,151,152,149,146,143,149,149,
+	142,141,151,152,149,146,143,151,
+	151,152,152,142,142,141,141,143,
+	143,139,141,128,128,128,128,128,
+};
 unsigned char frame_restrict[48] = {
 	0, 0, 0, 0, 0, 179, 186, 186,
 	205, 0, 0, 0, 0, 186, 205, 0,
@@ -461,6 +472,7 @@ unsigned char frame_restrict[48] = {
 		}									\
 		if (mode && c >= 176 && c < 224) c = frame_vt100[c - 176];		\
 	} else if (s->mode == TERM_KOI8 && (ch >> 15) && c >= 176 && c < 224) { c = frame_koi[c - 176];\
+	} else if (s->mode == TERM_FREEBSD && (ch >> 15) && c >= 176 && c < 224) { c = frame_freebsd[c - 176];\
 	} else if (s->mode == TERM_DUMB && (ch >> 15) && c >= 176 && c < 224) c = frame_dumb[c - 176];\
 	if (!(A & 0100) && (A >> 3) == (A & 7)) A = (A & 070) | 7 * !(A & 020);		\
 	if (A != attrib) {								\
@@ -524,6 +536,7 @@ void redraw_screen(struct terminal *term)
 			}
 		}
 	if (l) {
+		if (s->col) add_to_str(&a, &l, "\033[37;40m");
 		add_to_str(&a, &l, "\033[0m");
 		if (s->mode == TERM_LINUX && s->m11_hack) add_to_str(&a, &l, "\033[10m");
 		if (s->mode == TERM_VT100) add_to_str(&a, &l, "\x0f");
@@ -679,7 +692,7 @@ void exec_thread(unsigned char *path, int p)
 	if (path[0] == 2) setpgid(0, 0);
 #endif
 	exe(path + 1);
-	close(p);
+	/*close(p);*/
 	if (path[1 + strlen(path + 1) + 1]) unlink(path + 1 + strlen(path + 1) + 1);
 }
 
