@@ -8,16 +8,18 @@ struct {
 	int free_syntax;
 	int need_slashes;
 	int need_slash_after_host;
+	int allow_post;
 } protocols[]= {
-		{"file", 0, file_func, NULL, 1, 1, 0},
-		{"https", 443, https_func, NULL, 0, 1, 1},
-		{"http", 80, http_func, NULL, 0, 1, 1},
-		{"proxy", 3128, proxy_func, NULL, 0, 1, 1},
-		{"ftp", 21, ftp_func, NULL, 0, 1, 1},
-		{"finger", 79, finger_func, NULL, 0, 1, 1},
-		{"mailto", 0, NULL, mailto_func, 0, 0, 0},
-		{"telnet", 0, NULL, telnet_func, 0, 0, 0},
-		{"tn3270", 0, NULL, tn3270_func, 0, 0, 0},
+		{"file", 0, file_func, NULL, 1, 1, 0, 0},
+		{"https", 443, https_func, NULL, 0, 1, 1, 1},
+		{"http", 80, http_func, NULL, 0, 1, 1, 1},
+		{"proxy", 3128, proxy_func, NULL, 0, 1, 1, 1},
+		{"ftp", 21, ftp_func, NULL, 0, 1, 1, 0},
+		{"finger", 79, finger_func, NULL, 0, 1, 1, 0},
+		{"smb", 139, smb_func, NULL, 0, 1, 1, 0},
+		{"mailto", 0, NULL, mailto_func, 0, 0, 0, 0},
+		{"telnet", 0, NULL, telnet_func, 0, 0, 0, 0},
+		{"tn3270", 0, NULL, tn3270_func, 0, 0, 0, 0},
 		{NULL, 0, NULL}
 };
 
@@ -31,7 +33,7 @@ int check_protocol(unsigned char *p, int l)
 	return -1;
 }
 
-int get_prot_info(unsigned char *prot, int *port, void (**func)(struct connection *), void (**nc_func)(struct session *ses, unsigned char *))
+int get_prot_info(unsigned char *prot, int *port, void (**func)(struct connection *), void (**nc_func)(struct session *ses, unsigned char *), int *allow_post)
 {
 	int i;
 	for (i = 0; protocols[i].prot; i++)
@@ -39,6 +41,7 @@ int get_prot_info(unsigned char *prot, int *port, void (**func)(struct connectio
 			if (port) *port = protocols[i].port;
 			if (func) *func = protocols[i].func;
 			if (nc_func) *nc_func = protocols[i].nc_func;
+			if (allow_post) *allow_post = protocols[i].allow_post;
 			return 0;
 		}
 	return -1;
@@ -96,6 +99,7 @@ int parse_url(unsigned char *url, int *prlen, unsigned char **user, int *uslen,
 	if (*q == ':') {
 		unsigned char *pp = q + strcspn(q, "/");
 		int cc;
+		if (*pp != '/' && protocols[a].need_slash_after_host) return -1;
 		if (port) *port = q + 1;
 		if (polen) *polen = pp - q - 1;
 		for (cc = 0; cc < pp - q - 1; cc++) if (q[cc+1] < '0' || q[cc+1] > '9') return -1;
@@ -172,7 +176,7 @@ int get_port(unsigned char *url)
 		if (n) return n;
 	}
 	if ((h = get_protocol_name(url))) {
-		get_prot_info(h, &n, NULL, NULL);
+		get_prot_info(h, &n, NULL, NULL, NULL);
 		mem_free(h);
 	}
 	return n;
@@ -182,9 +186,11 @@ void (*get_protocol_handle(unsigned char *url))(struct connection *)
 {
 	unsigned char *p;
 	void (*f)(struct connection *) = NULL;
+	int post = 0;
 	if (!(p = get_protocol_name(url))) return NULL;
-	get_prot_info(p, NULL, &f, NULL);
+	get_prot_info(p, NULL, &f, NULL, &post);
 	mem_free(p);
+	if (!post && strchr(url, POST_CHAR)) return NULL;
 	return f;
 }
 
@@ -192,9 +198,11 @@ void (*get_external_protocol_function(unsigned char *url))(struct session *, uns
 {
 	unsigned char *p;
 	void (*f)(struct session *, unsigned char *) = NULL;
+	int post = 0;
 	if (!(p = get_protocol_name(url))) return NULL;
-	get_prot_info(p, NULL, NULL, &f);
+	get_prot_info(p, NULL, NULL, &f, &post);
 	mem_free(p);
+	if (!post && strchr(url, POST_CHAR)) return NULL;
 	return f;
 }
 
@@ -271,7 +279,9 @@ void translate_directories(unsigned char *url)
 		/*d = d1;
 		goto p;*/
 		b:
-		s += 3;
+		*d++ = *s;
+		if (s[3]) s += 4;
+		else s += 3;
 		goto r;
 	}
 	p:
