@@ -136,7 +136,6 @@ void copy_vs(struct view_state *dst, struct view_state *src)
 		memcpy(dst->form_info, src->form_info, src->form_info_len * sizeof(struct form_state));
 		for (i = 0; i < src->form_info_len; i++) if (src->form_info[i].value) dst->form_info[i].value = stracpy(src->form_info[i].value);
 	}
-	dst->f = NULL;
 }
 
 void copy_location(struct location *dst, struct location *src)
@@ -202,6 +201,7 @@ void sort_links(struct f_data *f)
 		int p, q, j;
 		struct link *link = &f->links[i];
 		if (!link->n) {
+			if (d_opt->num_links) continue;
 			if (link->where) mem_free(link->where);
 			if (link->target) mem_free(link->target);
 			if (link->where_img) mem_free(link->where_img);
@@ -465,7 +465,7 @@ void get_searched(struct f_data_c *scr, struct point **pt, int *pl)
 				co = get_char(t, x, y);
 				co = ((co >> 3) & 0x0700) | ((co << 3) & 0x3800);
 				set_color(t, x, y, co);*/
-				if (!(len & ALLOC_GR)) {
+				if (!(len & (ALLOC_GR - 1)) && !(len & (ALLOC_GR - 1)) && !(len & (ALLOC_GR - 1)) && !(len & (ALLOC_GR - 1)) /* obsessive-compulsive disorder rulez !!! */) {
 					struct point *npt;
 					if (!(npt = mem_realloc(points, sizeof(struct point) * (len + ALLOC_GR)))) continue;
 					points = npt;
@@ -696,6 +696,21 @@ void draw_forms(struct terminal *t, struct f_data_c *f)
 	} while (l1++ < l2);
 }
 
+/* 0 -> 1 <- 2 v 3 ^ */
+
+unsigned char fr_trans[2][4] = {{0xb3, 0xc3, 0xb4, 0xc5}, {0xc4, 0xc2, 0xc1, 0xc5}};
+
+void set_xchar(struct terminal *t, int x, int y, unsigned dir)
+{
+	unsigned c;
+	if (x < 0 || x >= t->x || y < 0 || y >= t->y) return;
+	c = get_char(t, x, y);
+	if (!(c & ATTR_FRAME)) return;
+	c &= 0xff;
+	if (c == fr_trans[dir / 2][0]) set_only_char(t, x, y, fr_trans[dir / 2][1 + (dir & 1)] | ATTR_FRAME);
+	else if (c == fr_trans[dir / 2][2 - (dir & 1)]) set_only_char(t, x, y, fr_trans[dir / 2][3] | ATTR_FRAME);
+}
+
 void draw_frame_lines(struct terminal *t, struct frameset_desc *fsd, int xp, int yp)
 {
 	int i, j;
@@ -707,9 +722,25 @@ void draw_frame_lines(struct terminal *t, struct frameset_desc *fsd, int xp, int
 		x = xp - 1;
 		for (i = 0; i < fsd->x; i++) {
 			int wwx = fsd->f[i].xw;
-			if (i) fill_area(t, x, y + 1, 1, wwy, 179 | ATTR_FRAME);
-			if (j) fill_area(t, x + 1, y, wwx, 1, 196 | ATTR_FRAME);
+			if (i) {
+				fill_area(t, x, y + 1, 1, wwy, 179 | ATTR_FRAME);
+				if (j == fsd->y - 1) set_xchar(t, x, y + wwy + 1, 3);
+			} else if (j) set_xchar(t, x, y, 0);
+			if (j) {
+				fill_area(t, x + 1, y, wwx, 1, 196 | ATTR_FRAME);
+				if (i == fsd->x - 1) set_xchar(t, x + wwx + 1, y, 1);
+			} else if (i) set_xchar(t, x, y, 2);
 			if (i && j) set_char(t, x, y, 197 | ATTR_FRAME);
+			x += wwx + 1;
+		}
+		y += wwy + 1;
+	}
+	y = yp - 1;
+	for (j = 0; j < fsd->y; j++) {
+		int wwy = fsd->f[j * fsd->x].yw;
+		x = xp - 1;
+		for (i = 0; i < fsd->x; i++) {
+			int wwx = fsd->f[i].xw;
 			if (fsd->f[j * fsd->x + i].subframe) {
 				draw_frame_lines(t, fsd->f[j * fsd->x + i].subframe, x + 1, y + 1);
 			}
@@ -1273,6 +1304,7 @@ void encode_multipart(struct session *ses, struct list_head *l, unsigned char **
 			if (*sv->value) {
 				if (anonymous) goto error;
 				if ((fh = open(sv->value, O_RDONLY)) == -1) goto error;
+				set_bin(fh);
 				do {
 					if ((rd = read(fh, buffer, F_BUFLEN)) == -1) goto error;
 					if (rd) add_bytes_to_str(data, len, buffer, rd);
@@ -1926,9 +1958,8 @@ struct f_data_c *current_frame(struct session *ses)
 		if (fd->f_data && fd->f_data->frame) continue;
 		if (!i--) return fd;
 	}
-	fd = cur_loc(ses)->vs.f;
-	if (fd->f_data && fd->f_data->frame) return NULL;
-	return fd;
+	if (!ses->screen || !ses->screen->f_data || ses->screen->f_data->frame) return NULL;
+	return ses->screen;
 }
 
 int send_to_frame(struct session *ses, struct event *ev)
@@ -2003,8 +2034,6 @@ void do_mouse_event(struct session *ses, struct event *ev)
 	send_to_frame(ses, &evv);
 }
 
-void head_msg(struct session *ses);
-
 void send_event(struct session *ses, struct event *ev)
 {
 	if (ev->ev == EV_KBD) {
@@ -2461,9 +2490,9 @@ void loc_msg(struct terminal *term, struct location *lo, struct f_data_c *frame)
 		add_to_str(&s, &l, "\n");
 		add_to_str(&s, &l, _(TEXT(T_CODEPAGE), term));
 		add_to_str(&s, &l, ": ");
-		add_to_str(&s, &l, get_cp_name(lo->vs.f->f_data->cp));
-		if (lo->vs.f->f_data->ass == 1) add_to_str(&s, &l, " ("), add_to_str(&s, &l, _(TEXT(T_ASSUMED), term)), add_to_str(&s, &l, ")");
-		if (lo->vs.f->f_data->ass == 2) add_to_str(&s, &l, " ("), add_to_str(&s, &l, _(TEXT(T_IGNORING_SERVER_SETTING), term)), add_to_str(&s, &l, ")");
+		add_to_str(&s, &l, get_cp_name(frame->f_data->cp));
+		if (frame->f_data->ass == 1) add_to_str(&s, &l, " ("), add_to_str(&s, &l, _(TEXT(T_ASSUMED), term)), add_to_str(&s, &l, ")");
+		if (frame->f_data->ass == 2) add_to_str(&s, &l, " ("), add_to_str(&s, &l, _(TEXT(T_IGNORING_SERVER_SETTING), term)), add_to_str(&s, &l, ")");
 		if ((a = parse_http_header(ce->head, "Server", NULL))) {
 			add_to_str(&s, &l, "\n");
 			add_to_str(&s, &l, _(TEXT(T_SERVER), term));
